@@ -1,6 +1,6 @@
 """HTTP Client for asyncio."""
 
-__all__ = ['request', 'HttpClient']
+__all__ = ['request', 'HttpClient', 'BasicAuth']
 
 import asyncio
 import base64
@@ -10,9 +10,6 @@ import json
 import io
 import inspect
 import itertools
-import logging
-import mimetypes
-import os
 import random
 import time
 import uuid
@@ -21,11 +18,15 @@ import weakref
 import warnings
 
 import aiohttp
+from . import helpers
+from .log import client_log
 from .multidict import CaseInsensitiveMultiDict, MultiDict, MutableMultiDict
 
 HTTP_PORT = 80
 HTTPS_PORT = 443
 
+BasicAuth = collections.namedtuple('BasicAuth', ['login', 'password'])
+
 
 @asyncio.coroutine
 def request(method, url, *,
@@ -60,7 +61,7 @@ def request(method, url, *,
     :param cookies: (optional) Dict object to send with the request
     :param files: (optional) Dictionary of 'name': file-like-objects
        for multipart encoding upload
-    :param auth: (optional) Auth tuple to enable Basic HTTP Auth
+    :param auth: (optional) BasicAuth named tuple represent HTTP Basic Auth
     :param allow_redirects: (optional) Boolean. Set to True if POST/PUT/DELETE
        redirect following is allowed.
     :param compress: Boolean. Set to True if request has to be compressed
@@ -68,7 +69,7 @@ def request(method, url, *,
     :param chunked: Boolean or Integer. Set to chunk size for chunked
        transfer encoding.
     :param expect100: Boolean. Expect 100-continue response from server.
-    :param connector: aiohttp.conntect.BaseConnector instance to support
+    :param connector: aiohttp.connector.BaseConnector instance to support
        connection pooling and session cookies.
     :param read_until_eof: Read response until eof if response
        does not have Content-Length header.
@@ -82,7 +83,7 @@ def request(method, url, *,
       >>> resp = yield from aiohttp.request('GET', 'http://python.org/')
       >>> resp
       <ClientResponse(python.org/) [200]>
-      >>> data = yield from resp.read_and_close()
+      >>> data = yield from resp.read()
 
     """
     redirects = 0
@@ -91,15 +92,14 @@ def request(method, url, *,
     if request_class is None:
         request_class = ClientRequest
     if connector is None:
-        connector = aiohttp.TCPConnector(loop=loop)
+        connector = aiohttp.TCPConnector(force_close=True, loop=loop)
 
     while True:
         req = request_class(
             method, url, params=params, headers=headers, data=data,
-            cookies=cookies, files=files, auth=auth, encoding=encoding,
+            cookies=cookies, files=files, encoding=encoding,
             version=version, compress=compress, chunked=chunked,
-            loop=loop, expect100=expect100,
-            response_class=response_class)
+            loop=loop, expect100=expect100, response_class=response_class)
 
         try:
             conn = yield from connector.connect(req)
@@ -120,20 +120,21 @@ def request(method, url, *,
         if resp.status in (301, 302) and allow_redirects:
             redirects += 1
             if max_redirects and redirects >= max_redirects:
-                resp.close()
+                resp.close(force=True)
                 break
 
             r_url = resp.headers.get('LOCATION') or resp.headers.get('URI')
 
             scheme = urllib.parse.urlsplit(r_url)[0]
             if scheme not in ('http', 'https', ''):
+                resp.close(force=True)
                 raise ValueError('Can redirect only to http or https')
             elif not scheme:
                 r_url = urllib.parse.urljoin(url, r_url)
 
             url = urllib.parse.urldefrag(r_url)[0]
             if url:
-                resp.close()
+                yield from asyncio.async(resp.release(), loop=loop)
                 continue
 
         break
@@ -143,8 +144,8 @@ def request(method, url, *,
 
 class ClientRequest:
 
-    GET_METHODS = {'DELETE', 'GET', 'HEAD', 'OPTIONS'}
-    POST_METHODS = {'PATCH', 'POST', 'PUT', 'TRACE'}
+    GET_METHODS = {'GET', 'HEAD', 'OPTIONS'}
+    POST_METHODS = {'PATCH', 'POST', 'PUT', 'TRACE', 'DELETE'}
     ALL_METHODS = GET_METHODS.union(POST_METHODS)
 
     DEFAULT_HEADERS = {
@@ -172,8 +173,8 @@ class ClientRequest:
     def __init__(self, method, url, *,
                  params=None, headers=None, data=None, cookies=None,
                  files=None, auth=None, encoding='utf-8', version=(1, 1),
-                 compress=None, chunked=None, expect100=False,
-                 verify_ssl=True, loop=None, response_class=None):
+                 compress=None, chunked=None, expect100=False, verify_ssl=True,
+                 loop=None, response_class=None):
         self.url = url
         self.method = method.upper()
         self.encoding = encoding
@@ -215,9 +216,11 @@ class ClientRequest:
         # basic auth info
         if '@' in netloc:
             authinfo, netloc = netloc.split('@', 1)
-            self.auth = authinfo.split(':', 1)
-            if len(self.auth) == 1:
-                self.auth.append('')
+            creds = authinfo.split(':', 1)
+            if len(creds) > 1:
+                self.auth = BasicAuth(creds[0], creds[1])
+            else:
+                self.auth = BasicAuth(creds[0], '')
 
         # Record entire netloc for usage in host header
         self.netloc = netloc
@@ -337,18 +340,24 @@ class ClientRequest:
 
     def update_auth(self, auth):
         """Set basic auth."""
-        if not auth:
+        if auth is None:
             auth = self.auth
+        if auth is None:
+            return
 
-        if auth:
-            if isinstance(auth, (tuple, list)) and len(auth) == 2:
-                # basic auth
-                self.headers['AUTHORIZATION'] = 'Basic %s' % (
-                    base64.b64encode(
-                        ('%s:%s' % (auth[0], auth[1])).encode('latin1'))
-                    .strip().decode('latin1'))
-            else:
-                raise ValueError("Only basic auth is supported")
+        if not isinstance(auth, BasicAuth):
+            warnings.warn(
+                'BasicAuth() tuple is required instead ', DeprecationWarning)
+
+        basic_login, basic_passwd = auth
+
+        if basic_login is not None and basic_passwd is not None:
+            self.headers['AUTHORIZATION'] = 'Basic %s' % (
+                base64.b64encode(
+                    ('%s:%s' % (basic_login, basic_passwd)).encode('latin1'))
+                .strip().decode('latin1'))
+        elif basic_login is not None or basic_passwd is not None:
+            raise ValueError("HTTP Auth login or password is missing")
 
     def update_body_from_data(self, data):
         if (hasattr(data, '__iter__') and not isinstance(
@@ -391,7 +400,7 @@ class ClientRequest:
                 data = data.items()
 
             for field, val in data:
-                fields.append((field, str_to_bytes(val)))
+                fields.append((field, helpers.str_to_bytes(val)))
 
         if isinstance(files, dict):
             files = list(files.items())
@@ -402,23 +411,23 @@ class ClientRequest:
 
             ft = None
             if len(rec) == 1:
-                k = guess_filename(rec[0], 'unknown')
+                k = helpers.guess_filename(rec[0], 'unknown')
                 fields.append((k, k, rec[0]))
 
             elif len(rec) == 2:
                 k, fp = rec
-                fn = guess_filename(fp, k)
+                fn = helpers.guess_filename(fp, k)
                 fields.append((k, fn, fp))
 
             else:
                 k, fp, ft = rec
-                fn = guess_filename(fp, k)
+                fn = helpers.guess_filename(fp, k)
                 fields.append((k, fn, fp, ft))
 
         self.chunked = self.chunked or 8192
         boundary = uuid.uuid4().hex
 
-        self.body = encode_multipart_data(
+        self.body = helpers.encode_multipart_data(
             fields, bytes(boundary, 'latin1'))
 
         self.headers['CONTENT-TYPE'] = (
@@ -621,8 +630,11 @@ class ClientResponse:
             self.message.headers.items(getall=True))
 
         # payload
+        response_with_body = self.method.lower() != 'head'
         self.content = self._reader.set_parser(
-            aiohttp.HttpPayloadParser(self.message, readall=read_until_eof))
+            aiohttp.HttpPayloadParser(self.message,
+                                      readall=read_until_eof,
+                                      response_with_body=response_with_body))
 
         # cookies
         self.cookies = http.cookies.SimpleCookie()
@@ -631,7 +643,8 @@ class ClientResponse:
                 try:
                     self.cookies.load(hdr)
                 except http.cookies.CookieError as exc:
-                    logging.warn('Can not load response cookies: %s', exc)
+                    client_log.warning(
+                        'Can not load response cookies: %s', exc)
 
         return self
 
@@ -648,6 +661,10 @@ class ClientResponse:
             self._writer = None
             self._writer_wr = None
 
+    @asyncio.coroutine
+    def release(self):
+        yield from self.read()
+
     @asyncio.coroutine
     def wait_for_close(self):
         if self._writer is not None:
@@ -671,7 +688,10 @@ class ClientResponse:
                     buf.append((chunk, size))
                     total += size
             except aiohttp.EofStream:
-                pass
+                self.close()
+            except:
+                self.close(True)
+                raise
 
             self._content = bytearray(total)
 
@@ -684,97 +704,51 @@ class ClientResponse:
         data = self._content
 
         if decode:
-            ct = self.headers.get('CONTENT-TYPE', '').lower()
-            if ct == 'application/json':
-                data = json.loads(data.decode('utf-8'))
+            warnings.warn(
+                '.read(True) is deprecated. use .json() instead',
+                DeprecationWarning
+            )
+            return (yield from self.json())
 
         return data
 
     @asyncio.coroutine
     def read_and_close(self, decode=False):
         """Read response payload and then close response."""
-        try:
-            payload = yield from self.read(decode)
-        except:
-            self.close(True)
-            raise
-        else:
-            self.close()
+        warnings.warn(
+            'read_and_close is deprecated, use .read() instead',
+            DeprecationWarning
+        )
+        return (yield from self.read(decode))
 
-        return payload
+    @asyncio.coroutine
+    def json(self, *, encoding=None, loads=json.loads):
+        """Reads and decodes JSON response."""
+        if self._content is None:
+            yield from self.read()
 
+        ctype = self.headers.get('CONTENT-TYPE', '').lower()
+        mtype, stype, _, params = helpers.parse_mimetype(ctype)
+        if not (mtype == 'application' or stype == 'json'):
+            client_log.warning(
+                'Attempt to decode JSON with unexpected mimetype: %s', ctype)
 
-def str_to_bytes(s, encoding='utf-8'):
-    if isinstance(s, str):
-        return s.encode(encoding)
-    return s
+        if not self._content.strip():
+            return None
 
-
-def guess_filename(obj, default=None):
-    name = getattr(obj, 'name', None)
-    if name and name[0] != '<' and name[-1] != '>':
-        return os.path.split(name)[-1]
-    return default
-
-
-def encode_multipart_data(fields, boundary, encoding='utf-8', chunk_size=8196):
-    """
-    Encode a list of fields using the multipart/form-data MIME format.
-
-    fields:
-        List of (name, value) or (name, filename, io) or
-        (name, filename, io, MIME type) field tuples.
-    """
-    for rec in fields:
-        yield b'--' + boundary + b'\r\n'
-
-        field, *rec = rec
-
-        if len(rec) == 1:
-            data = rec[0]
-            yield (('Content-Disposition: form-data; name="%s"\r\n\r\n' %
-                    (field,)).encode(encoding))
-            yield data + b'\r\n'
-
-        else:
-            if len(rec) == 3:
-                fn, fp, ct = rec
-            else:
-                fn, fp = rec
-                ct = (mimetypes.guess_type(fn)[0] or
-                      'application/octet-stream')
-
-            yield ('Content-Disposition: form-data; name="%s"; '
-                   'filename="%s"\r\n' % (field, fn)).encode(encoding)
-            yield ('Content-Type: %s\r\n\r\n' % (ct,)).encode(encoding)
-
-            if isinstance(fp, str):
-                fp = fp.encode(encoding)
-
-            if isinstance(fp, bytes):
-                fp = io.BytesIO(fp)
-
-            while True:
-                chunk = fp.read(chunk_size)
-                if not chunk:
-                    break
-                yield str_to_bytes(chunk)
-
-            yield b'\r\n'
-
-    yield b'--' + boundary + b'--\r\n'
+        encoding = encoding or params.get('charset', 'utf-8')
+        return loads(self._content.decode(encoding))
 
 
 class HttpClient:
-    """Allow to use mutiple hosts with same path. And automatically
+    """Allow to use multiple hosts with same path. And automatically
     mark failed hosts.
     """
 
-    def __init__(self, hosts, *, method=None, path=None,
-                 ssl=False,
+    def __init__(self, hosts, *, method=None, path=None, ssl=False,
                  conn_pool=True, conn_timeout=None, failed_timeout=5.0,
-                 resolve=True, resolve_timeout=360.0, verify_ssl=True,
-                 loop=None):
+                 resolve=True, resolve_timeout=360.0, keepalive_timeout=30,
+                 verify_ssl=True, loop=None):
         super().__init__()
 
         if isinstance(hosts, str):
@@ -817,6 +791,7 @@ class HttpClient:
         if conn_pool:
             self._connector = aiohttp.TCPConnector(
                 share_cookies=True, conn_timeout=conn_timeout,
+                keepalive_timeout=keepalive_timeout,
                 resolve=resolve, verify_ssl=verify_ssl, loop=loop)
 
             self._resolve_timeout = resolve_timeout
