@@ -43,13 +43,14 @@ class Connection(object):
 
 class BaseConnector(object):
 
-    def __init__(self, *, reuse_timeout=30, conn_timeout=None,
-                 share_cookies=False, loop=None, **kwargs):
+    def __init__(self, *, conn_timeout=None, keepalive_timeout=30,
+                 share_cookies=False, force_close=False, loop=None, **kwargs):
         self._conns = {}
-        self._reuse_timeout = reuse_timeout
         self._conn_timeout = conn_timeout
+        self._keepalive_timeout = keepalive_timeout
         self._share_cookies = share_cookies
         self._cleanup_handle = None
+        self._force_close = force_close
 
         if loop is None:
             loop = asyncio.get_event_loop()
@@ -57,9 +58,8 @@ class BaseConnector(object):
         self._factory = functools.partial(aiohttp.StreamProtocol, loop=loop)
 
         self.cookies = http.cookies.SimpleCookie()
-        self._wr = weakref.ref(self,
-                               lambda wr, f=self._do_close, conns=self._conns:
-                               f(conns))
+        self._wr = weakref.ref(
+            self, lambda wr, f=self._do_close, conns=self._conns: f(conns))
 
     def _cleanup(self):
         """Cleanup unused transports."""
@@ -76,7 +76,7 @@ class BaseConnector(object):
                 if transport is not None:
                     if proto and not proto.is_connected():
                         transport = None
-                    elif (now - t0) > self._reuse_timeout:
+                    elif (now - t0) > self._keepalive_timeout:
                         transport.close()
                         transport = None
 
@@ -87,17 +87,16 @@ class BaseConnector(object):
 
         if connections:
             self._cleanup_handle = self._loop.call_later(
-                self._reuse_timeout, self._cleanup)
+                self._keepalive_timeout, self._cleanup)
 
         self._conns = connections
-        self._wr = weakref.ref(self,
-                               lambda wr, f=self._do_close, conns=self._conns:
-                               f(conns))
+        self._wr = weakref.ref(
+            self, lambda wr, f=self._do_close, conns=self._conns: f(conns))
 
     def _start_cleanup_task(self):
         if self._cleanup_handle is None:
             self._cleanup_handle = self._loop.call_later(
-                self._reuse_timeout, self._cleanup)
+                self._keepalive_timeout, self._cleanup)
 
     def close(self):
         """Close all opened transports."""
@@ -145,7 +144,7 @@ class BaseConnector(object):
         while conns:
             transport, proto, t0 = conns.pop()
             if transport is not None and proto.is_connected():
-                if (time.time() - t0) > self._reuse_timeout:
+                if (time.time() - t0) > self._keepalive_timeout:
                     transport.close()
                     transport = None
                 else:
@@ -165,6 +164,9 @@ class BaseConnector(object):
                 if self._share_cookies and resp.cookies:
                     self.update_cookies(resp.cookies.items())
 
+        if self._force_close:
+            should_close = True
+
         reader = protocol.reader
         if should_close or (reader.output and not reader.output.at_eof()):
             transport.close()
@@ -192,6 +194,26 @@ class TCPConnector(BaseConnector):
         self._resolve = resolve
         self._resolved_hosts = {}
 
+    @property
+    def verify_ssl(self):
+        """Do check for ssl certifications?"""
+        return self._verify_ssl
+
+    @property
+    def family(self):
+        """Socket family like AF_INET"""
+        return self._family
+
+    @property
+    def resolve(self):
+        """Do DNS lookup for host name?"""
+        return self._resolve
+
+    @property
+    def resolved_hosts(self):
+        """The dict of (host, port) -> (ipaddr, port) pairs"""
+        return dict(self._resolved_hosts)
+
     def clear_resolved_hosts(self, host=None, port=None):
         if host is not None and port is not None:
             key = (host, port)
@@ -252,29 +274,37 @@ class TCPConnector(BaseConnector):
 class ProxyConnector(TCPConnector):
     """Http Proxy connector."""
 
-    def __init__(self, proxy, *args, **kwargs):
+    def __init__(self, proxy, proxy_login=None, proxy_passwd=None,
+                 *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.proxy = proxy
+        self._proxy = proxy
+        self._basic_login = proxy_login
+        self._basic_passwd = proxy_passwd
         assert proxy.startswith('http://'), (
             "Only http proxy supported", proxy)
 
+    @property
+    def proxy(self):
+        return self._proxy
+
     @asyncio.coroutine
     def _create_connection(self, req, **kwargs):
-        proxy_req = ClientRequest('GET', self.proxy,
-                                  headers={'Host': req.host},
-                                  loop=self._loop)
+        proxy_req = ClientRequest(
+            'GET', self._proxy,
+            headers={'Host': req.host},
+            auth=aiohttp.BasicAuth(self._basic_login, self._basic_passwd),
+            loop=self._loop)
         try:
             transport, proto = yield from super()._create_connection(proxy_req)
-        except OSError:
-            raise ProxyConnectionError()
+        except OSError as exc:
+            raise ProxyConnectionError(*exc.args) from exc
         req.path = '{scheme}://{host}{path}'.format(scheme=req.scheme,
                                                     host=req.host,
                                                     path=req.path)
-        if proxy_req.auth:
+        if 'AUTHORIZATION' in proxy_req.headers:
             auth = proxy_req.headers['AUTHORIZATION']
             del proxy_req.headers['AUTHORIZATION']
             req.headers['PROXY-AUTHORIZATION'] = auth
-            proxy_req.headers['PROXY-AUTHORIZATION'] = auth
 
         if req.ssl:
             # For HTTPS requests over HTTP proxy
@@ -317,12 +347,17 @@ class UnixConnector(BaseConnector):
     def __init__(self, path, *args, **kw):
         super().__init__(*args, **kw)
 
-        self.path = path
+        self._path = path
+
+    @property
+    def path(self):
+        """Path to unix socket"""
+        return self._path
 
     @asyncio.coroutine
     def _create_connection(self, req, **kwargs):
         return (yield from self._loop.create_unix_connection(
-            self._factory, self.path, **kwargs))
+            self._factory, self._path, **kwargs))
 
 
 SocketConnector = TCPConnector
