@@ -7,6 +7,7 @@ import time
 import unittest
 import unittest.mock
 import urllib.parse
+import socket
 
 import aiohttp
 from aiohttp.client import ClientRequest, ClientResponse, HttpClient
@@ -51,7 +52,8 @@ class ClientResponseTests(unittest.TestCase):
 
         connection = unittest.mock.Mock()
         response._setup_connection(connection)
-        del response
+        with self.assertWarns(ResourceWarning):
+            del response
 
         connection.close.assert_called_with()
 
@@ -78,29 +80,119 @@ class ClientResponseTests(unittest.TestCase):
             '<ClientResponse(http://python.org) [200 Ok]>',
             repr(self.response))
 
-    def test_read_and_close(self):
-        self.response.read = unittest.mock.Mock()
-        self.response.read.return_value = asyncio.Future(loop=self.loop)
-        self.response.read.return_value.set_result(b'payload')
+    def test_read_and_release_connection(self):
+        def side_effect(*args, **kwargs):
+            def second_call(*args, **kwargs):
+                raise aiohttp.EofStream
+            fut = asyncio.Future(loop=self.loop)
+            fut.set_result(b'payload')
+            content.read.side_effect = second_call
+            return fut
+        content = self.response.content = unittest.mock.Mock()
+        content.read.side_effect = side_effect
         self.response.close = unittest.mock.Mock()
 
-        res = self.loop.run_until_complete(self.response.read_and_close())
+        res = self.loop.run_until_complete(self.response.read())
         self.assertEqual(res, b'payload')
-        self.assertTrue(self.response.read.called)
         self.assertTrue(self.response.close.called)
 
-    def test_read_and_close_with_error(self):
-        self.response.read = unittest.mock.Mock()
-        self.response.read.return_value = asyncio.Future(loop=self.loop)
-        self.response.read.return_value.set_exception(ValueError)
+    def test_read_and_release_connection_with_error(self):
+        content = self.response.content = unittest.mock.Mock()
+        content.read.return_value = asyncio.Future(loop=self.loop)
+        content.read.return_value.set_exception(ValueError)
         self.response.close = unittest.mock.Mock()
 
         self.assertRaises(
             ValueError,
-            self.loop.run_until_complete, self.response.read_and_close())
-        self.assertTrue(self.response.read.called)
+            self.loop.run_until_complete, self.response.read())
         self.response.close.assert_called_with(True)
 
+    def test_release(self):
+        fut = asyncio.Future(loop=self.loop)
+        fut.set_exception(aiohttp.EofStream)
+        content = self.response.content = unittest.mock.Mock()
+        content.read.return_value = fut
+        self.response.close = unittest.mock.Mock()
+
+        self.loop.run_until_complete(self.response.release())
+        self.assertTrue(self.response.close.called)
+
+    def test_read_and_close(self):
+        self.response.read = unittest.mock.Mock()
+        self.response.read.return_value = asyncio.Future(loop=self.loop)
+        self.response.read.return_value.set_result(b'data')
+
+        res = self.loop.run_until_complete(self.response.read_and_close())
+        self.assertEqual(res, b'data')
+        self.assertTrue(self.response.read.called)
+
+    def test_read_decode_deprecated(self):
+        self.response._content = b'data'
+        self.response.json = unittest.mock.Mock()
+        self.response.json.return_value = asyncio.Future(loop=self.loop)
+        self.response.json.return_value.set_result('json')
+
+        res = self.loop.run_until_complete(self.response.read(decode=True))
+        self.assertEqual(res, 'json')
+        self.assertTrue(self.response.json.called)
+
+    def test_json(self):
+        def side_effect(*args, **kwargs):
+            def second_call(*args, **kwargs):
+                raise aiohttp.EofStream
+            fut = asyncio.Future(loop=self.loop)
+            fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
+            content.read.side_effect = second_call
+            return fut
+        self.response.headers = {
+            'CONTENT-TYPE': 'application/json;charset=cp1251'}
+        content = self.response.content = unittest.mock.Mock()
+        content.read.side_effect = side_effect
+        self.response.close = unittest.mock.Mock()
+
+        res = self.loop.run_until_complete(self.response.json())
+        self.assertEqual(res, {'тест': 'пройден'})
+        self.assertTrue(self.response.close.called)
+
+    def test_json_custom_loader(self):
+        self.response.headers = {
+            'CONTENT-TYPE': 'application/json;charset=cp1251'}
+        self.response._content = b'data'
+
+        def custom(content):
+            return content + '-custom'
+
+        res = self.loop.run_until_complete(self.response.json(loads=custom))
+        self.assertEqual(res, 'data-custom')
+
+    def test_json_no_content(self):
+        self.response.headers = {
+            'CONTENT-TYPE': 'data/octet-stream'}
+        self.response._content = b''
+        self.response.close = unittest.mock.Mock()
+
+        res = self.loop.run_until_complete(self.response.json())
+        self.assertIsNone(res)
+
+    def test_json_override_encoding(self):
+        def side_effect(*args, **kwargs):
+            def second_call(*args, **kwargs):
+                raise aiohttp.EofStream
+            fut = asyncio.Future(loop=self.loop)
+            fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
+            content.read.side_effect = second_call
+            return fut
+        self.response.headers = {
+            'CONTENT-TYPE': 'application/json;charset=utf8'}
+        content = self.response.content = unittest.mock.Mock()
+        content.read.side_effect = side_effect
+        self.response.close = unittest.mock.Mock()
+
+        res = self.loop.run_until_complete(
+            self.response.json(encoding='cp1251'))
+        self.assertEqual(res, {'тест': 'пройден'})
+        self.assertTrue(self.response.close.called)
+
 
 class ClientRequestTests(unittest.TestCase):
 
@@ -205,6 +297,12 @@ class ClientRequestTests(unittest.TestCase):
         self.assertEqual('/', req.path)
 
     def test_basic_auth(self):
+        req = ClientRequest('get', 'http://python.org',
+                            auth=aiohttp.BasicAuth('nkim', '1234'))
+        self.assertIn('AUTHORIZATION', req.headers)
+        self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
+
+    def test_basic_auth_tuple_deprecated(self):
         req = ClientRequest('get', 'http://python.org', auth=('nkim', '1234'))
         self.assertIn('AUTHORIZATION', req.headers)
         self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
@@ -214,19 +312,17 @@ class ClientRequestTests(unittest.TestCase):
         self.assertIn('AUTHORIZATION', req.headers)
         self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
 
-        req = ClientRequest('get', 'http://nkim@python.org')
-        self.assertIn('AUTHORIZATION', req.headers)
-        self.assertEqual('Basic bmtpbTo=', req.headers['AUTHORIZATION'])
-
         req = ClientRequest(
-            'get', 'http://nkim@python.org', auth=('nkim', '1234'))
+            'get', 'http://nkim@python.org',
+            auth=aiohttp.BasicAuth('nkim', '1234'))
         self.assertIn('AUTHORIZATION', req.headers)
         self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
 
     def test_basic_auth_err(self):
+        # missing password here
         self.assertRaises(
             ValueError, ClientRequest,
-            'get', 'http://python.org', auth=(1, 2, 3))
+            'get', 'http://python.org', auth=aiohttp.BasicAuth('nkim', None))
 
     def test_no_content_length(self):
         req = ClientRequest('get', 'http://python.org', loop=self.loop)
@@ -637,7 +733,17 @@ class HttpClientTests(unittest.TestCase):
         c = HttpClient(
             [('localhost', 56777), ('localhost', 56778)], loop=self.loop)
         c._hosts = []
-        c._failed.append((('localhost', 1000, False), now - 10))
+
+        has_http_server = True
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        try:
+            sock.connect(('127.0.0.1', 80))
+        except ConnectionError:
+            has_http_server = False
+        finally:
+            sock.close()
+
+        c._failed.append((('localhost', 1000, has_http_server), now - 10))
         c._failed.append((('localhost', 1001, True), now - 10))
 
         self.assertRaises(
