@@ -7,6 +7,7 @@ import json
 import http.cookies
 import asyncio
 import unittest
+from unittest import mock
 
 import aiohttp
 from aiohttp import client
@@ -38,7 +39,10 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 content = content1.decode()
 
                 self.assertEqual(r.status, 200)
-                self.assertIn('"method": "%s"' % meth.upper(), content)
+                if meth == 'head':
+                    self.assertEqual(b'', content1)
+                else:
+                    self.assertIn('"method": "%s"' % meth.upper(), content)
                 self.assertEqual(content1, content2)
                 r.close()
 
@@ -55,7 +59,10 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 content = content1.decode()
 
                 self.assertEqual(r.status, 200)
-                self.assertIn('"method": "%s"' % meth.upper(), content)
+                if meth == 'head':
+                    self.assertEqual(b'', content1)
+                else:
+                    self.assertIn('"method": "%s"' % meth.upper(), content)
                 self.assertEqual(content1, content2)
                 r.close()
 
@@ -73,7 +80,10 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 content = content1.decode()
 
                 self.assertEqual(r.status, 200)
-                self.assertIn('"method": "%s"' % meth.upper(), content)
+                if meth == 'head':
+                    self.assertEqual(b'', content1)
+                else:
+                    self.assertIn('"method": "%s"' % meth.upper(), content)
                 self.assertEqual(content1, content2)
                 r.close()
 
@@ -91,7 +101,10 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 content = content1.decode()
 
                 self.assertEqual(r.status, 200)
-                self.assertIn('"method": "%s"' % meth.upper(), content)
+                if meth == 'head':
+                    self.assertEqual(b'', content1)
+                else:
+                    self.assertIn('"method": "%s"' % meth.upper(), content)
                 self.assertEqual(content1, content2)
                 r.close()
 
@@ -116,14 +129,26 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
         with test_utils.run_server(self.loop, use_ssl=True) as httpd:
             for meth in ('get', 'post', 'put', 'delete', 'head'):
-                r = self.loop.run_until_complete(
-                    client.request(meth, httpd.url('method', meth),
-                                   loop=self.loop, connector=connector))
-                content = self.loop.run_until_complete(r.read())
+                @asyncio.coroutine
+                def go():
+                    yield from asyncio.sleep(0.1, loop=self.loop)
 
-                self.assertEqual(r.status, 200)
-                self.assertEqual(content, b'Test message')
-                r.close()
+                    r = yield from client.request(
+                        meth, httpd.url('method', meth),
+                        loop=self.loop, connector=connector)
+                    content = yield from r.read()
+
+                    self.assertEqual(r.status, 200)
+                    if meth == 'head':
+                        self.assertEqual(b'', content)
+                    else:
+                        self.assertEqual(content, b'Test message')
+                    r.close()
+                    # let loop to make one iteration to call connection_lost
+                    # and close socket
+                    yield from asyncio.sleep(0, loop=self.loop)
+
+                self.loop.run_until_complete(go())
 
     def test_use_global_loop(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
@@ -144,21 +169,27 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
     def test_HTTP_302_REDIRECT_GET(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
-            r = self.loop.run_until_complete(
-                client.request('get', httpd.url('redirect', 2),
-                               loop=self.loop))
+            @asyncio.coroutine
+            def go():
+                r = yield from client.request('get',
+                                              httpd.url('redirect', 2),
+                                              loop=self.loop)
 
-            self.assertEqual(r.status, 200)
-            self.assertEqual(2, httpd['redirects'])
-            r.close()
+                self.assertEqual(r.status, 200)
+                self.assertEqual(2, httpd['redirects'])
+                r.close()
+            self.loop.run_until_complete(go())
 
     def test_HTTP_302_REDIRECT_NON_HTTP(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
-            self.assertRaises(
-                ValueError,
-                self.loop.run_until_complete,
-                client.request('get', httpd.url('redirect_err'),
-                               loop=self.loop))
+            @asyncio.coroutine
+            def go():
+                with self.assertRaises(ValueError):
+                    yield from client.request('get',
+                                              httpd.url('redirect_err'),
+                                              loop=self.loop)
+
+            self.loop.run_until_complete(go())
 
     def test_HTTP_302_REDIRECT_POST(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
@@ -197,16 +228,21 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
     def test_HTTP_200_GET_WITH_MIXED_PARAMS(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
-            r = self.loop.run_until_complete(
-                client.request(
+            @asyncio.coroutine
+            def go():
+                r = yield from client.request(
                     'get', httpd.url('method', 'get') + '?test=true',
-                    params={'q': 'test'}, loop=self.loop))
-            content = self.loop.run_until_complete(r.content.read())
-            content = content.decode()
+                    params={'q': 'test'}, loop=self.loop)
+                content = yield from r.content.read()
+                content = content.decode()
 
-            self.assertIn('"query": "test=true&q=test"', content)
-            self.assertEqual(r.status, 200)
-            r.close()
+                self.assertIn('"query": "test=true&q=test"', content)
+                self.assertEqual(r.status, 200)
+                r.close()
+                # let loop to make one iteration to call connection_lost
+                # and close socket
+                yield from asyncio.sleep(0, loop=self.loop)
+            self.loop.run_until_complete(go())
 
     def test_POST_DATA(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
@@ -216,7 +252,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                                loop=self.loop))
             self.assertEqual(r.status, 200)
 
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             self.assertEqual({'some': ['data']}, content['form'])
             self.assertEqual(r.status, 200)
             r.close()
@@ -230,7 +266,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                                loop=self.loop))
             self.assertEqual(r.status, 200)
 
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             self.assertEqual('deflate', content['compression'])
             self.assertEqual({'some': ['data']}, content['form'])
             self.assertEqual(r.status, 200)
@@ -246,7 +282,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                         'post', url, files={'some': f}, chunked=1024,
                         headers={'Transfer-Encoding': 'chunked'},
                         loop=self.loop))
-                content = self.loop.run_until_complete(r.read(True))
+                content = self.loop.run_until_complete(r.json())
 
                 f.seek(0)
                 filename = os.path.split(f.name)[-1]
@@ -271,7 +307,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                                    chunked=1024, compress='deflate',
                                    loop=self.loop))
 
-                content = self.loop.run_until_complete(r.read(True))
+                content = self.loop.run_until_complete(r.json())
 
                 f.seek(0)
                 filename = os.path.split(f.name)[-1]
@@ -296,7 +332,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                     client.request('post', url, files=[('some', f.read())],
                                    loop=self.loop))
 
-                content = self.loop.run_until_complete(r.read(True))
+                content = self.loop.run_until_complete(r.json())
 
                 f.seek(0)
                 self.assertEqual(1, len(content['multipart-data']))
@@ -318,7 +354,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                     client.request('post', url, files=[('some', f)],
                                    loop=self.loop))
 
-                content = self.loop.run_until_complete(r.read(True))
+                content = self.loop.run_until_complete(r.json())
 
                 f.seek(0)
                 filename = os.path.split(f.name)[-1]
@@ -342,7 +378,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                     client.request('post', url, loop=self.loop,
                                    files=[('some', f, 'text/plain')]))
 
-                content = self.loop.run_until_complete(r.read(True))
+                content = self.loop.run_until_complete(r.json())
 
                 f.seek(0)
                 filename = os.path.split(f.name)[-1]
@@ -367,7 +403,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 r = self.loop.run_until_complete(
                     client.request('post', url, files=[f], loop=self.loop))
 
-                content = self.loop.run_until_complete(r.read(True))
+                content = self.loop.run_until_complete(r.json())
 
                 f.seek(0)
                 filename = os.path.split(f.name)[-1]
@@ -391,7 +427,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
             r = self.loop.run_until_complete(
                 client.request('post', url, files=[data], loop=self.loop))
 
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
 
             self.assertEqual(1, len(content['multipart-data']))
             self.assertEqual(
@@ -411,7 +447,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                     client.request('post', url, loop=self.loop,
                                    data={'test': 'true'}, files={'some': f}))
 
-                content = self.loop.run_until_complete(r.read(True))
+                content = self.loop.run_until_complete(r.json())
 
                 self.assertEqual(2, len(content['multipart-data']))
                 self.assertEqual(
@@ -451,7 +487,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                     'post', url, data=stream(),
                     headers={'Content-Length': str(len(data))},
                     loop=self.loop))
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             r.close()
 
             self.assertEqual(str(len(data)),
@@ -465,7 +501,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                                expect100=True, loop=self.loop))
             self.assertEqual(r.status, 200)
 
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             self.assertEqual('100-continue', content['headers']['Expect'])
             self.assertEqual(r.status, 200)
             r.close()
@@ -498,7 +534,8 @@ class HttpClientFunctionalTests(unittest.TestCase):
             self.assertIn(b'"Cookie": "test1=123; test3=456"', bytes(content))
             r.close()
 
-    def test_set_cookies(self):
+    @mock.patch('aiohttp.client.client_log')
+    def test_set_cookies(self, m_log):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
             resp = self.loop.run_until_complete(
                 client.request('get', httpd.url('cookies'), loop=self.loop))
@@ -509,13 +546,16 @@ class HttpClientFunctionalTests(unittest.TestCase):
             self.assertEqual(resp.cookies['c2'].value, 'cookie2')
             resp.close()
 
+        m_log.warning.assert_called_with('Can not load response cookies: %s',
+                                         mock.ANY)
+
     def test_chunked(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
             r = self.loop.run_until_complete(
                 client.request('get', httpd.url('chunked'), loop=self.loop))
             self.assertEqual(r.status, 200)
             self.assertEqual(r.headers.getone('TRANSFER-ENCODING'), 'chunked')
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             self.assertEqual(content['path'], '/chunked')
             r.close()
 
@@ -526,7 +566,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
             self.assertEqual(r.status, 200)
             self.assertRaises(
                 aiohttp.IncompleteRead,
-                self.loop.run_until_complete, r.read(True))
+                self.loop.run_until_complete, r.json())
             r.close()
 
     def test_request_conn_error(self):
@@ -554,7 +594,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 client.request('get', httpd.url('keepalive',),
                                connector=c, loop=self.loop))
             self.assertEqual(r.status, 200)
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             self.assertEqual(content['content'], 'requests=1')
             r.close()
 
@@ -562,10 +602,12 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 client.request('get', httpd.url('keepalive'),
                                connector=c, loop=self.loop))
             self.assertEqual(r.status, 200)
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             self.assertEqual(content['content'], 'requests=2')
             r.close()
 
+        c.close()
+
     def test_session_close(self):
         conn = aiohttp.TCPConnector(loop=self.loop)
 
@@ -575,7 +617,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                     'get', httpd.url('keepalive') + '?close=1',
                     connector=conn, loop=self.loop))
             self.assertEqual(r.status, 200)
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             self.assertEqual(content['content'], 'requests=1')
             r.close()
 
@@ -583,11 +625,14 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 client.request('get', httpd.url('keepalive'),
                                connector=conn, loop=self.loop))
             self.assertEqual(r.status, 200)
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             self.assertEqual(content['content'], 'requests=1')
             r.close()
 
-    def test_session_cookies(self):
+        conn.close()
+
+    @mock.patch('aiohttp.client.client_log')
+    def test_session_cookies(self, m_log):
         from aiohttp import connector
         conn = connector.TCPConnector(share_cookies=True, loop=self.loop)
 
@@ -597,7 +642,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 client.request('get', httpd.url('cookies'),
                                connector=conn, loop=self.loop))
             self.assertEqual(r.status, 200)
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
 
             self.assertEqual(content['headers']['Cookie'], 'test=1')
             r.close()
@@ -605,6 +650,8 @@ class HttpClientFunctionalTests(unittest.TestCase):
             cookies = sorted([(k, v.value) for k, v in conn.cookies.items()])
             self.assertEqual(
                 cookies, [('c1', 'cookie1'), ('c2', 'cookie2'), ('test', '1')])
+        m_log.warning.assert_called_with('Can not load response cookies: %s',
+                                         mock.ANY)
 
     def test_multidict_headers(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
@@ -617,12 +664,46 @@ class HttpClientFunctionalTests(unittest.TestCase):
                     'post', url, data=data,
                     headers=MultiDict({'Content-Length': str(len(data))}),
                     loop=self.loop))
-            content = self.loop.run_until_complete(r.read(True))
+            content = self.loop.run_until_complete(r.json())
             r.close()
 
             self.assertEqual(str(len(data)),
                              content['headers']['Content-Length'])
 
+    def test_close_implicit_connector(self):
+
+        @asyncio.coroutine
+        def go(url):
+            r = yield from client.request('GET', url, loop=self.loop)
+
+            connection = r.connection
+            self.assertIsNotNone(connection)
+            connector = connection._connector
+            self.assertIsNotNone(connector)
+            yield from r.read()
+            self.assertEqual(0, len(connector._conns))
+
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('keepalive')
+            self.loop.run_until_complete(go(url))
+
+    def test_dont_close_explicit_connector(self):
+
+        @asyncio.coroutine
+        def go(url):
+            connector = aiohttp.TCPConnector(loop=self.loop)
+
+            r = yield from client.request('GET', url,
+                                          connector=connector,
+                                          loop=self.loop)
+            yield from r.read()
+            self.assertEqual(1, len(connector._conns))
+            connector.close()
+
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('keepalive')
+            self.loop.run_until_complete(go(url))
+
 
 class Functional(test_utils.Router):
 
