@@ -201,7 +201,7 @@ class BaseConnectorTests(unittest.TestCase):
         conn._start_cleanup_task()
         self.assertIsNotNone(conn._cleanup_handle)
         loop.call_later.assert_called_with(
-            conn._reuse_timeout, conn._cleanup)
+            conn._keepalive_timeout, conn._cleanup)
 
     @unittest.mock.patch('aiohttp.connector.time')
     def test_cleanup(self, time):
@@ -235,6 +235,13 @@ class BaseConnectorTests(unittest.TestCase):
 
         self.assertIsNotNone(conn._cleanup_handle)
 
+    def test_tcp_connector_ctor(self):
+        conn = aiohttp.TCPConnector(loop=self.loop)
+        self.assertTrue(conn.verify_ssl)
+        self.assertFalse(conn.resolve)
+        self.assertEqual(conn.family, socket.AF_INET)
+        self.assertEqual(conn.resolved_hosts, {})
+
 
 class HttpClientConnectorTests(unittest.TestCase):
 
@@ -265,13 +272,15 @@ class HttpClientConnectorTests(unittest.TestCase):
     def test_unix_connector(self):
         path = '/tmp/aiohttp_unix.sock'
 
+        connector = aiohttp.UnixConnector(path, loop=self.loop)
+        self.assertEqual(path, connector.path)
+
         with test_utils.run_server(
                 self.loop, listen_addr=path, router=Functional) as httpd:
             r = self.loop.run_until_complete(
                 client.request(
                     'get', httpd.url('method', 'get'),
-                    connector=aiohttp.UnixConnector(
-                        path, loop=self.loop),
+                    connector=connector,
                     loop=self.loop))
             content = self.loop.run_until_complete(r.content.read())
             content = content.decode()
@@ -305,6 +314,12 @@ class ProxyConnectorTests(unittest.TestCase):
         with self.assertRaises(AssertionError):
             aiohttp.ProxyConnector('https://localhost:8118', loop=self.loop)
 
+    def test_ctor2(self):
+        connector = aiohttp.ProxyConnector('http://localhost:8118',
+                                           loop=self.loop)
+
+        self.assertEqual('http://localhost:8118', connector.proxy)
+
     @unittest.mock.patch('aiohttp.connector.ClientRequest')
     def test_connect(self, ClientRequestMock):
         req = ClientRequest('GET', 'http://www.python.org')
@@ -331,6 +346,7 @@ class ProxyConnectorTests(unittest.TestCase):
 
         ClientRequestMock.assert_called_with(
             'GET', 'http://proxy.example.com',
+            auth=aiohttp.BasicAuth(None, None),
             headers={'Host': 'www.python.org'},
             loop=loop_mock)
 
@@ -349,6 +365,35 @@ class ProxyConnectorTests(unittest.TestCase):
 
     @unittest.mock.patch('aiohttp.connector.ClientRequest')
     def test_auth(self, ClientRequestMock):
+        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+                                  auth=aiohttp.BasicAuth('user', 'pass'))
+        ClientRequestMock.return_value = proxy_req
+        self.assertIn('AUTHORIZATION', proxy_req.headers)
+        self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
+
+        loop_mock = unittest.mock.Mock()
+        connector = aiohttp.ProxyConnector(
+            'http://proxy.example.com', loop=loop_mock,
+            proxy_login='user', proxy_passwd='pass')
+        connector._resolve_host = resolve_mock = unittest.mock.Mock()
+        self._fake_coroutine(resolve_mock, [unittest.mock.MagicMock()])
+
+        tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
+        self._fake_coroutine(loop_mock.create_connection, (tr, proto))
+
+        req = ClientRequest('GET', 'http://www.python.org')
+        self.assertNotIn('AUTHORIZATION', req.headers)
+        self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
+        self.loop.run_until_complete(connector.connect(req))
+
+        self.assertEqual(req.path, 'http://www.python.org/')
+        self.assertNotIn('AUTHORIZATION', req.headers)
+        self.assertIn('PROXY-AUTHORIZATION', req.headers)
+        self.assertNotIn('AUTHORIZATION', proxy_req.headers)
+        self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
+
+    @unittest.mock.patch('aiohttp.connector.ClientRequest')
+    def test_auth_from_url(self, ClientRequestMock):
         proxy_req = ClientRequest('GET', 'http://user:pass@proxy.example.com')
         ClientRequestMock.return_value = proxy_req
         self.assertIn('AUTHORIZATION', proxy_req.headers)
@@ -372,7 +417,7 @@ class ProxyConnectorTests(unittest.TestCase):
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertIn('PROXY-AUTHORIZATION', req.headers)
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
-        self.assertIn('PROXY-AUTHORIZATION', proxy_req.headers)
+        self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
 
     @unittest.mock.patch('aiohttp.connector.ClientRequest')
     def test_auth__not_modifying_request(self, ClientRequestMock):
