@@ -4,6 +4,7 @@ import asyncio
 import unittest
 import unittest.mock
 
+from aiohttp import errors
 from aiohttp import parsers
 from aiohttp import test_utils
 
@@ -167,8 +168,8 @@ class StreamParserTests(unittest.TestCase):
         stream.feed_data(b'line1')
         stream.feed_eof()
         s = stream.set_parser(p)
-        self.assertIsNone(s.exception())
-        self.assertTrue(s._eof)
+        self.assertFalse(s.at_eof())
+        self.assertIsInstance(s.exception(), errors.ConnectionError)
 
     def test_set_parser_unset(self):
         stream = parsers.StreamParser(paused=False)
@@ -213,7 +214,7 @@ class StreamParserTests(unittest.TestCase):
         self.assertEqual([bytearray(b'line1\r\n'), bytearray(b'line2\r\n')],
                          list(s._buffer))
         self.assertEqual(b'data', bytes(stream._buffer))
-        self.assertTrue(s._eof)
+        self.assertTrue(s.at_eof())
 
     def test_feed_parser_exc(self):
         def p(out, buf):
@@ -280,8 +281,8 @@ class StreamParserTests(unittest.TestCase):
 
         stream.feed_data(b'line1')
         stream.feed_eof()
-        self.assertIsNone(s.exception())
-        self.assertTrue(s._eof)
+        self.assertFalse(s.at_eof())
+        self.assertIsInstance(s.exception(), errors.ConnectionError)
 
     def test_feed_parser2(self):
         stream = parsers.StreamParser()
@@ -321,8 +322,8 @@ class StreamParserTests(unittest.TestCase):
 
         stream.feed_data(b'line1')
         stream.unset_parser()
-        self.assertIsNone(s.exception(), ValueError)
-        self.assertTrue(s._eof)
+        self.assertIsInstance(s.exception(), errors.ConnectionError)
+        self.assertFalse(s.at_eof())
 
     def test_unset_parser_stop(self):
         def p(out, buf):
@@ -427,12 +428,33 @@ class DataQueueTests(unittest.TestCase):
 
     def test_read_exception(self):
         buffer = parsers.DataQueue(self.stream, loop=self.loop)
-        buffer.feed_data(object())
         buffer.set_exception(ValueError())
 
         self.assertRaises(
             ValueError, self.loop.run_until_complete, buffer.read())
 
+    def test_read_exception_with_data(self):
+        val = object()
+        buffer = parsers.DataQueue(self.stream, loop=self.loop)
+        buffer.feed_data(val)
+        buffer.set_exception(ValueError())
+
+        self.assertIs(val, self.loop.run_until_complete(buffer.read()))
+        self.assertRaises(
+            ValueError, self.loop.run_until_complete, buffer.read())
+
+    def test_read_exception_on_wait(self):
+        buffer = parsers.DataQueue(self.stream, loop=self.loop)
+        read_task = asyncio.Task(buffer.read(), loop=self.loop)
+        test_utils.run_briefly(self.loop)
+        self.assertIsInstance(buffer._waiter, asyncio.Future)
+
+        buffer.feed_eof()
+        buffer.set_exception(ValueError())
+
+        self.assertRaises(
+            ValueError, self.loop.run_until_complete, read_task)
+
     def test_exception(self):
         buffer = parsers.DataQueue(self.stream, loop=self.loop)
         self.assertIsNone(buffer.exception())
@@ -612,25 +634,19 @@ class ParserBufferTests(unittest.TestCase):
         next(p)
         p.send(b'1')
         p.send(b'234')
-        self.assertRaises(ValueError, p.send, b'5')
+        self.assertRaises(errors.LineLimitExceededParserError, p.send, b'5')
 
         buf = parsers.ParserBuffer()
         p = buf.readuntil(b'\n', 4)
         next(p)
-        self.assertRaises(ValueError, p.send, b'12345\n6')
+        self.assertRaises(
+            errors.LineLimitExceededParserError, p.send, b'12345\n6')
 
         buf = parsers.ParserBuffer()
         p = buf.readuntil(b'\n', 4)
         next(p)
-        self.assertRaises(ValueError, p.send, b'12345\n6')
-
-        class CustomExc(Exception):
-            pass
-
-        buf = parsers.ParserBuffer()
-        p = buf.readuntil(b'\n', 4, CustomExc)
-        next(p)
-        self.assertRaises(CustomExc, p.send, b'12345\n6')
+        self.assertRaises(
+            errors.LineLimitExceededParserError, p.send, b'12345\n6')
 
     def test_readuntil(self):
         buf = self._make_one()
@@ -651,25 +667,19 @@ class ParserBufferTests(unittest.TestCase):
         next(p)
         p.send(b'1')
         p.send(b'234')
-        self.assertRaises(ValueError, p.send, b'5')
+        self.assertRaises(errors.LineLimitExceededParserError, p.send, b'5')
 
         buf = parsers.ParserBuffer()
         p = buf.waituntil(b'\n', 4)
         next(p)
-        self.assertRaises(ValueError, p.send, b'12345\n6')
+        self.assertRaises(
+            errors.LineLimitExceededParserError, p.send, b'12345\n6')
 
         buf = parsers.ParserBuffer()
         p = buf.waituntil(b'\n', 4)
         next(p)
-        self.assertRaises(ValueError, p.send, b'12345\n6')
-
-        class CustomExc(Exception):
-            pass
-
-        buf = parsers.ParserBuffer()
-        p = buf.waituntil(b'\n', 4, CustomExc)
-        next(p)
-        self.assertRaises(CustomExc, p.send, b'12345\n6')
+        self.assertRaises(
+            errors.LineLimitExceededParserError, p.send, b'12345\n6')
 
     def test_waituntil(self):
         buf = self._make_one()
@@ -716,7 +726,7 @@ class ParserBufferTests(unittest.TestCase):
             list(out._buffer))
         try:
             p.throw(parsers.EofStream())
-        except parsers.EofStream:
+        except StopIteration:
             pass
 
         self.assertEqual(bytes(buf), b'data')
@@ -734,7 +744,7 @@ class ParserBufferTests(unittest.TestCase):
             [bytearray(b'line1'), bytearray(b'line2')], list(out._buffer))
         try:
             p.throw(parsers.EofStream())
-        except parsers.EofStream:
+        except StopIteration:
             pass
 
         self.assertEqual(bytes(buf), b'data')
